/**
* The class responsible for opening and retreiving values
* form a locally or remotely stoped file
* or from an applet
* @author Randy Levensalor
* @version 3.0
*/

package form.io;

import java.io.*;
import java.awt.FileDialog;
import java.awt.Frame;
import java.applet.Applet;
import java.net.*;

public class Open implements Serializable {

	/**
	* The reader for the input stream from the file
	*/
	private BufferedReader inReader = null;

	/**
	* String containing the the entire file
	* only used for LOCAL and FTP opens
	* null if applet
	*/
	private String data = null;

	/**
	* The title of the open dialog for local files
	*/
	private final static String OPEN_LOCAL_TITLE = "Open local file";

	/**
	* The type of file being opened
	* Must be initialized in constructor and 
	* not changes after words
	*/
	private int type;

	public final static int LOCAL  = 0;
	public final static int FTP    = 1;
	public final static int APPLET = 2;

	/**
	* Only used when type == APPLET
	* Must be initialized in constructor if used
	*/
	private Applet applet = null;


	/**
	* Checks if the file was open sucessfully
	*/
	public boolean isOpen(){
	   return (data != null);
	}

	private void openLocal(Frame frame){
	   FileDialog fd = new FileDialog(frame,OPEN_LOCAL_TITLE,FileDialog.LOAD);
	   String fileName = null;

	   fd.show();
	   if (fd.getDirectory()!=null){
	      fileName = fd.getDirectory()+fd.getFile();
	   }

	   //if open was cancled will not open file
	   //can be checked with isOpen()
	   if (fileName != null)
	     openLocal(fileName);
	}

	private void openLocal(String fileName){

	   try {
		 FileReader f = new FileReader(fileName);
		 inReader = new BufferedReader(f);
	   } catch (IOException e) { System.out.println("Cannot open file "+fileName); }

	   read();
	}

	private void openFtp(Frame frame){

	   FtpDialog fd = new FtpDialog(frame,FtpDialog.OPEN);

	   inReader = fd.getReader();

	   read();
	}

	private void read(){
	  String nextLine = "";
	  data = "";

	  try{
	    while ( (nextLine = inReader.readLine()) !=null ) {

	      data +=nextLine;
	    }
	  } catch (IOException e) {
	     System.err.println("Cannot read input stream");
	     data = null;
	     inReader = null;
	  }

	  try {
	    inReader.close();

	  } catch (IOException e) { 
	    System.err.println("Cannot close input stream");
	    data = null;
	    inReader = null;
	  }

	}

	 
	public String getParameter(String pattern){
	   String parameter = null;

	   if (type == APPLET) {
	      parameter = applet.getParameter(pattern);
	   }
	   //type == LOCAL || FTP
	   else {
	      int next = 0;
	      int nextEnd = 0;
	      String temp;

	      //next = the index of the patter followed by = 
	      // folowed by ="
	      next = data.indexOf(pattern+"\"");

	      //patern not found
	      if (next <0) 
	         return null;

	      //move the next index to the end of the pattern
	      next += pattern.length()+2;
	      next += data.substring(next).indexOf("\"")+1;

	      //find the end of the parameter
	      nextEnd = data.substring(next).indexOf("\"");

	      parameter = data.substring(next,next+nextEnd);
	   }
	   return parameter;
	}

	/**
	* Open a file locally or via ftp
	* @param int type The type to opened either open.LOCAL or open.FTP
	* @param Frame frame used for open dialog
	*/
	public Open(int type, Frame frame){
	    if ( type == LOCAL){
	       this.type = type;
	       openLocal(frame);
	    }
	    else if ( type == FTP){
	       this.type = type;
	       openFtp(frame);
	    }
	    else {
	       System.out.println("Open failed to initialized.  Invalid open type");
	    }
	}

	/**
	* Open a file locally or via ftp
	* @param String fileName The name of the file to be opened
	*  if fileName == ftp will start ftp dialog
	*  used when file name is specified at command line
	* @param Frame frame used for open dialog
	*/
	public Open(String fileName,Frame frame){
	    if ( fileName == null ) {
	       System.out.println("Open failed to initialized.  Null name");
	    }
	    else if ( fileName.equals("ftp") ){
	       this.type = FTP;
	       openFtp(frame);
	    }
	    else {
	       this.type = LOCAL;
	       openLocal(fileName);
	    }
	}

	/**
	* Open using an Applet
	* @param Applet applet The applet this is running as
	*/
	public Open(Applet applet){
	   if (applet != null){
	      type = APPLET;
	      this.applet = applet;
	   }
	   else {
	      System.out.println("Open fialed Null applet passed.");
	   }
	}

	/**
	* Returns an int 
	* is unstable if it is not an int
	*/
	//should throw error if conversion fails
	public int getInt(String pattern){
	   int val = Integer.valueOf(getParameter(pattern)).intValue();
	   return val;
	}

	/**
	* Returns a double
	* is unstable if it is not a double
	*/
	//should throw error if conversion fails
	public double getDouble(String pattern){
	   double val = Double.valueOf(getParameter(pattern)).doubleValue();
	   return val;
	}

	/**
	* Returns a boolean
	*/
	public boolean getBoolean(String pattern){
	   String param = getParameter(pattern);
	   return ( (param != null) && (param.equals("true")) );
	}

}
